# IMPORT STANDARD
from enum import Enum


class RecordType(str, Enum):
    none = "none"
    all = "all"
    curl = "curl"
    http = "http"
    object = "object"
    screen = "screen"
    time = "time"



# IMPORT STANDARD

# IMPORT THIRD-PARTY
from _pytest.config.argparsing import Parser

# IMPORT INTERNAL
from pytest_recorder.record_type import RecordType


def pytest_addoption(parser: Parser) -> None:
    group = parser.getgroup("recorder")
    group.addoption(
        "--record",
        action="store",
        default=RecordType.none,
        choices=[item.name for item in RecordType],
        help="Records the listed elements (default: none).",
    )
    group.addoption(
        "--record-no-hash",
        action="store_true",
        default=False,
        help="Avoid hashing the content before saving the records, apply to : object and screen.",
    )
    group.addoption(
        "--record-no-overwrite",
        action="store_true",
        default=False,
        help="Avoid rewriting existing records, apply to : http, object, screen, time.",
    )



# IMPORT STANDARD
import json
import logging
from datetime import datetime
from pathlib import Path

# IMPORT THIRD-PARTY
import pytest
import time_machine
from _pytest.fixtures import SubRequest

# IMPORT INTERNAL
from pytest_recorder.record_type import RecordType

logger = logging.getLogger(__name__)


def pytest_configure(config):
    config.addinivalue_line(
        "markers",
        "record_time(destination, *, tick): Records time or time travel to the saved time.",
    )


class TravelModel:
    @property
    def dt(self) -> datetime:
        return self._dt

    @property
    def tick(self) -> bool:
        return self._tick

    def __init__(self, dt: datetime, tick: bool) -> None:
        self._dt = dt
        self._tick = tick


class TravelHandler:
    @staticmethod
    def load_travel(record_file_path: Path) -> TravelModel:
        if record_file_path.exists():
            logger.debug("Loading record file : %s", record_file_path)
            with record_file_path.open(
                mode="r", encoding="utf-8", newline="\n"
            ) as file:
                data = json.load(file)

            if isinstance(data, dict) and "isoformat" in data and "tick" in data:
                isoformat = data["isoformat"]
                tick = data["tick"]
                dt = datetime.fromisoformat(isoformat)
                travel = TravelModel(dt=dt, tick=tick)

                return travel
            else:
                raise AttributeError(
                    f"No JSON object with`isoformat` time found : {record_file_path}",
                )
        else:
            raise AttributeError(
                f"Cannot load record file : {record_file_path}",
            )

        return dt

    @staticmethod
    def persist(
        travel: TravelModel,
        record_file_path: Path,
    ):
        logger.debug("Making record folder : %s", record_file_path.parent)
        record_file_path.parent.mkdir(parents=True, exist_ok=True)

        data = {
            "isoformat": travel.dt.isoformat(),
            "tick": travel.tick,
        }
        with record_file_path.open(mode="w", encoding="utf-8", newline="\n") as file:
            logger.debug("Writing record file : %s", record_file_path)
            json.dump(data, file)


class RecordFilePathBuilder:
    @staticmethod
    def build(test_module_path: Path, test_function: str) -> Path:
        test_module = test_module_path.stem

        record_folder_name = RecordType.time.name
        record_file_folder_path = (
            test_module_path.parent / "record" / record_folder_name
        )
        record_file_name = f"{test_function}.json"
        record_file_path = record_file_folder_path / test_module / record_file_name

        return record_file_path


def record_time_context_manager(
    request: SubRequest,
):
    marker = request.node.get_closest_marker("record_time")

    if marker:
        record_no_overwrite = request.config.getoption("--record-no-overwrite")
        record_type = request.config.getoption("--record")
        test_function = request.node.name
        test_module_path = Path(request.node.fspath)  # PYTEST 6.2.2 COMPATIBILITY

        record_file_path = RecordFilePathBuilder.build(
            test_module_path=test_module_path,
            test_function=test_function,
        )

        if (RecordType.all in record_type or RecordType.time in record_type) and not (
            record_file_path.exists() and record_no_overwrite
        ):
            if len(marker.args) > 0 and isinstance(marker.args[0], datetime):
                dt = marker.args[0]
            else:
                dt = marker.kwargs.get("destination", datetime.now())

            tick = marker.kwargs.get("tick", False)
            travel = TravelModel(dt=dt, tick=tick)

            TravelHandler.persist(
                travel=travel,
                record_file_path=record_file_path,
            )
            travel_context_manager = time_machine.travel(
                destination=travel.dt,
                tick=travel.tick,
            )
        elif record_file_path.exists():
            travel = TravelHandler.load_travel(record_file_path=record_file_path)

            travel_context_manager = time_machine.travel(
                destination=travel.dt,
                tick=travel.tick,
            )
        else:
            raise AttributeError(
                f"No time to record to compare with the current result : {record_file_path}",
            )

        with travel_context_manager as traveller:
            yield traveller
    else:
        yield None


record_fixture = pytest.fixture(name="record_time", autouse=True)(
    record_time_context_manager
)



"""Pytest configuration and fixtures for YFinance tests."""

from pathlib import Path
from typing import Any
from unittest.mock import patch
import yaml
import pytest
import vcr
from _pytest.fixtures import SubRequest
from vcr.persisters.filesystem import FilesystemPersister, serialize
from curl_adapter import CurlCffiAdapter

from pytest_recorder.record_type import RecordType


@pytest.fixture(name="vcr_config")
def vcr_config_fixture() -> dict[str, Any]:
    return {}


def pytest_configure(config):
    config.addinivalue_line(
        "markers",
        "record_curl: Records HTTP request or the last recorded requests.",
    )


class VCRFilesystemPersister(FilesystemPersister):
    """Persister for curl cassettes."""

    @staticmethod
    def save_cassette(cassette_path, cassette_dict, serializer):
        data = serialize(cassette_dict, serializer)
        cassette_path = Path(cassette_path).resolve()
        cassette_path.parent.mkdir(parents=True, exist_ok=True)

        with cassette_path.open("w", encoding="utf-8", newline="\n") as f:
            f.write(data)


class CurlRecordFilePathBuilder:
    """Builds file paths for curl recordings."""

    @staticmethod
    def build(test_module_path: Path, test_function: str) -> Path:
        test_module = test_module_path.stem

        record_folder_name = RecordType.curl.name
        record_file_folder_path = (
            test_module_path.parent / "record" / record_folder_name
        )
        record_file_name = f"{test_function}_curl.yaml"
        record_file_path = record_file_folder_path / test_module / record_file_name

        return record_file_path


class CurlCassette:
    """Cassette-like object for curl requests."""

    def __init__(self, cassette_path, record_mode="once"):
        self.cassette_path = Path(cassette_path)
        self.record_mode = record_mode
        self.captured_requests = []
        self.existing_cassette = VCRFilesystemPersister.load_cassette(cassette_path)
        self._original_send = None

    def __enter__(self):
        if self._should_record():
            self._patch_curl_adapter()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self._should_record() and self.captured_requests:
            cassette_data = {"interactions": self.captured_requests}
            VCRFilesystemPersister.save_cassette(
                self.cassette_path, cassette_data, yaml.safe_dump
            )

        if self._original_send:
            # Restore original send method
            CurlCffiAdapter.send = self._original_send

    def _should_record(self):
        if self.record_mode == "none":
            return False
        elif self.record_mode == "once":
            return not self.cassette_path.exists()
        elif self.record_mode == "all":
            return True
        return False

    def _patch_curl_adapter(self):
        self._original_send = CurlCffiAdapter.send

        def capture_send(self_adapter, request, *args, **kwargs):
            # Call the original send method
            response = self._original_send(self_adapter, request, *args, **kwargs)

            # Create cassette-compatible request/response pair
            cassette_entry = {
                "request": {
                    "method": request.method,
                    "uri": str(request.url),
                    "headers": dict(request.headers),
                    "body": request.body.decode() if request.body else None,
                },
                "response": {
                    "status": {
                        "code": response.status_code,
                        "message": response.reason,
                    },
                    "headers": dict(response.headers),
                    "body": {"string": response.text, "encoding": "utf-8"},
                },
            }
            self.captured_requests.append(cassette_entry)

            return response

        CurlCffiAdapter.send = capture_send


def record_curl_context_manager(
    request: SubRequest,
    vcr_config: dict[str, Any],
):
    """Context manager for curl recording."""
    marker = request.node.get_closest_marker("record_curl")

    if marker:
        record_no_overwrite = request.config.getoption(
            "--record-no-overwrite", default=False
        )
        record_type = request.config.getoption("--record", default="none")
        test_function = request.node.name
        test_module_path = Path(request.node.fspath)

        record_file_path = CurlRecordFilePathBuilder.build(
            test_module_path=test_module_path,
            test_function=test_function,
        )

        def apply_vcr_filters(cassette_entry, vcr_config):
            """Apply VCR config filters to cassette entry."""
            # Filter headers
            if "filter_headers" in vcr_config:
                for header_name, replacement in vcr_config["filter_headers"]:
                    if header_name in cassette_entry["request"]["headers"]:
                        if replacement is None:
                            del cassette_entry["request"]["headers"][header_name]
                        else:
                            cassette_entry["request"]["headers"][
                                header_name
                            ] = replacement
                    if header_name in cassette_entry["response"]["headers"]:
                        if replacement is None:
                            del cassette_entry["response"]["headers"][header_name]
                        else:
                            cassette_entry["response"]["headers"][
                                header_name
                            ] = replacement

            # Filter query parameters
            if "filter_query_parameters" in vcr_config:
                from urllib.parse import urlparse, parse_qs, urlencode, urlunparse

                parsed_url = urlparse(cassette_entry["request"]["uri"])
                query_params = parse_qs(parsed_url.query)

                for param_name, replacement in vcr_config["filter_query_parameters"]:
                    if param_name in query_params:
                        query_params[param_name] = [replacement]

                # Rebuild URL with filtered parameters
                new_query = urlencode(query_params, doseq=True)
                new_parsed = parsed_url._replace(query=new_query)
                cassette_entry["request"]["uri"] = urlunparse(new_parsed)

            # Handle before_record_response function
            if "before_record_response" in vcr_config:
                for filter_func in vcr_config["before_record_response"]:
                    filtered_response = filter_func(cassette_entry["response"])

                    # If filter returns None, don't record this request at all
                    if filtered_response is None:
                        return None  # Don't record this request

                    # Apply the filtered response back to the cassette entry
                    cassette_entry["response"] = filtered_response

            return cassette_entry

        # Determine if we should record and create VCR object
        if (RecordType.all in record_type or RecordType.curl in record_type) and not (
            record_file_path.exists() and record_no_overwrite
        ):
            record_file_path.unlink(missing_ok=True)

            # Create VCR config without custom filters that VCR doesn't recognize
            vcr_config_clean = {
                k: v
                for k, v in vcr_config.items()
                if k not in ["before_record_response"]
            }

            # Create VCR object for recording
            vcr_object = vcr.VCR(
                cassette_library_dir=str(record_file_path.parent),
                record_mode="once",
                **vcr_config_clean,
            )
            vcr_object.register_persister(VCRFilesystemPersister)

            # Patch CurlCffiAdapter to capture requests
            captured_requests = []
            original_send = CurlCffiAdapter.send

            def capture_send(self_adapter, request, *args, **kwargs):
                response = original_send(self_adapter, request, *args, **kwargs)

                cassette_entry = {
                    "request": {
                        "method": request.method,
                        "uri": str(request.url),
                        "headers": dict(request.headers),
                        "body": request.body if request.body else None,
                    },
                    "response": {
                        "status": {
                            "code": response.status_code,
                            "message": response.reason,
                        },
                        "headers": dict(response.headers),
                        "body": {
                            "string": (
                                response.content
                                if response.status_code == 200
                                else None
                            ),
                            "encoding": "utf-8",
                        },
                    },
                }
                if cassette_entry["response"]["status"]["code"] == 200:
                    # Apply VCR config filters
                    cassette_entry = apply_vcr_filters(cassette_entry, vcr_config)

                    # Only add to captured_requests if the filter didn't return None
                    if cassette_entry is not None:
                        captured_requests.append(cassette_entry)
                return response

            with patch.object(CurlCffiAdapter, "send", capture_send):
                with vcr_object.use_cassette(record_file_path.name) as cassette:
                    yield cassette

            # Save the captured curl requests separately
            if captured_requests:
                cassette_data = {"interactions": captured_requests}
                record_file_path.parent.mkdir(parents=True, exist_ok=True)
                with record_file_path.open("w", encoding="utf-8") as f:
                    yaml.dump(cassette_data, f, default_flow_style=False)

        elif record_file_path.exists():
            # Use existing cassette
            vcr_object = vcr.VCR(
                cassette_library_dir=str(record_file_path.parent),
                record_mode="none",
                **vcr_config,
            )
            vcr_object.register_persister(VCRFilesystemPersister)

            with vcr_object.use_cassette(record_file_path.name) as cassette:
                yield cassette
        else:
            raise AttributeError(
                f"No comparison possible since there is no curl cassette: {record_file_path}",
            )
    else:
        yield None


# Create the fixture
record_curl_fixture = pytest.fixture(name="record_curl", autouse=True)(
    record_curl_context_manager
)



# IMPORT STANDARD
import json
import logging
from hashlib import sha256
from pathlib import Path

# IMPORT THIRD-PARTY
import pytest
from _pytest.capture import CaptureResult, MultiCapture, SysCapture
from _pytest.fixtures import SubRequest

# IMPORT INTERNAL
from pytest_recorder.record_type import RecordType

logger = logging.getLogger(__name__)


def pytest_configure(config):
    config.addinivalue_line(
        "markers",
        "record_verify_screen: record the text output on the screen and compare it on next run.",
    )


class CaptureResultHandler:
    @staticmethod
    def hash_output(capture_result: CaptureResult) -> CaptureResult:
        out = capture_result.out
        out_hash = sha256(out.encode()).hexdigest()
        err = capture_result.err
        capture_result_hash = CaptureResult(out=out_hash, err=err)

        return capture_result_hash

    @staticmethod
    def load_capture_result(record_file_path: Path) -> CaptureResult:
        if record_file_path.exists():
            logger.debug("Loading record file : %s", record_file_path)
            with record_file_path.open(
                mode="r", encoding="utf-8", newline="\n"
            ) as file:
                data = json.load(file)

            if isinstance(data, dict):
                out = data.get("out", "")
                err = data.get("err", "")
                capture_result = CaptureResult(out=out, err=err)
            else:
                out = ""
                err = ""
                capture_result = CaptureResult(out=out, err=err)
        else:
            raise AttributeError(
                f"Cannot load record file : {record_file_path}",
            )

        return capture_result

    @staticmethod
    def persist(
        capture_result: CaptureResult,
        record_file_path: Path,
    ):
        logger.debug("Making record folder : %s", record_file_path.parent)
        record_file_path.parent.mkdir(parents=True, exist_ok=True)

        data = {
            "out": capture_result.out,
            "err": capture_result.err,
        }
        with record_file_path.open(mode="w", encoding="utf-8", newline="\n") as file:
            logger.debug("Writing record file : %s", record_file_path)
            json.dump(data, file)

    @staticmethod
    def verify(
        capture_result: CaptureResult,
        capture_result_loaded: CaptureResult,
        record_file_path: Path,
    ):
        if capture_result.out != capture_result_loaded.out:
            raise AttributeError(
                "Recorded screen output doesn't match current screen output:\n"
                f"\nRECORD_FILE_PATH =\n{record_file_path}\n"
                f"\nCURRENT = {capture_result.out}\n"
                f"\nLOADED  = {capture_result_loaded.out}\n"
            )


class RecordFilePathBuilder:
    @staticmethod
    def build(test_module_path: Path, test_function: str) -> Path:
        test_module = test_module_path.stem

        record_folder_name = RecordType.screen.name
        record_file_folder_path = (
            test_module_path.parent / "record" / record_folder_name
        )
        record_file_name = f"{test_function}.json"
        record_file_path = record_file_folder_path / test_module / record_file_name

        return record_file_path


def record_screen_context_manager(
    request: SubRequest,
):
    marker = request.node.get_closest_marker("record_verify_screen")

    if marker:
        capture = request.config.getoption("--capture")
        record_no_hash = request.config.getoption("--record-no-hash")
        record_no_overwrite = request.config.getoption("--record-no-overwrite")
        record_type = request.config.getoption("--record")
        test_function = request.node.name
        test_module_path = Path(request.node.fspath)  # PYTEST 6.2.2 COMPATIBILITY

        record_file_path = RecordFilePathBuilder.build(
            test_module_path=test_module_path,
            test_function=test_function,
        )

        if capture == "no":
            global_capturing = MultiCapture(
                in_=SysCapture(0),
                out=SysCapture(1),
                err=SysCapture(2),
            )
            global_capturing.start_capturing()
            yield None
            capture_result = global_capturing.readouterr()
            global_capturing.stop_capturing()
        else:
            capsys = request.getfixturevalue("capsys")
            yield None
            capture_result = capsys.readouterr()

        if not record_no_hash:
            capture_result = CaptureResultHandler.hash_output(
                capture_result=capture_result
            )

        if (RecordType.all in record_type or RecordType.screen in record_type) and not (
            record_file_path.exists() and record_no_overwrite
        ):
            print("persist", capture_result)
            CaptureResultHandler.persist(
                capture_result=capture_result,
                record_file_path=record_file_path,
            )
        elif record_file_path.exists():
            capture_result_loaded = CaptureResultHandler.load_capture_result(
                record_file_path=record_file_path
            )
            CaptureResultHandler.verify(
                capture_result=capture_result,
                capture_result_loaded=capture_result_loaded,
                record_file_path=record_file_path,
            )
        else:
            raise AttributeError(
                f"No screen output recording to compare with the current result : {record_file_path}",
            )

    else:
        yield None


record_fixture = pytest.fixture(name="record_verify_screen", autouse=True)(
    record_screen_context_manager
)



# IMPORT STANDARD
import urllib3
from pathlib import Path
from typing import Any, Dict

# IMPORT THIRD-PARTY
import pytest
import vcr
from _pytest.fixtures import SubRequest
from vcr.persisters.filesystem import FilesystemPersister, serialize

# IMPORT INTERNAL
from pytest_recorder.record_type import RecordType


@pytest.fixture(name="vcr_config")
def vcr_config_fixture() -> Dict[str, Any]:
    return {}


def pytest_configure(config):
    config.addinivalue_line(
        "markers",
        "record_http: Records HTTP request or the last recorded requests.",
    )


class VCRFilesystemPersister(FilesystemPersister):
    @staticmethod
    def save_cassette(cassette_path, cassette_dict, serializer):
        data = serialize(cassette_dict, serializer)
        cassette_path = Path(cassette_path).resolve()
        cassette_path.parent.mkdir(parents=True, exist_ok=True)

        with cassette_path.open("w", encoding="utf-8", newline="\n") as f:
            f.write(data)


class RecordFilePathBuilder:
    @staticmethod
    def build(test_module_path: Path, test_function: str) -> Path:
        test_module = test_module_path.stem

        record_folder_name = RecordType.http.name
        record_file_folder_path = (
            test_module_path.parent / "record" / record_folder_name
        )
        urllib3_version = urllib3.__version__.split(".")[0]
        record_file_name = f"{test_function}_urllib3_v{urllib3_version}.yaml"
        record_file_path = record_file_folder_path / test_module / record_file_name

        return record_file_path


def record_http_context_manager(
    request: SubRequest,
    vcr_config: Dict[str, Any],
):
    marker = request.node.get_closest_marker("record_http")

    if marker:
        record_no_overwrite = request.config.getoption("--record-no-overwrite")
        record_type = request.config.getoption("--record")
        test_function = request.node.name
        test_module_path = Path(request.node.fspath)  # PYTEST 6.2.2 COMPATIBILITY

        record_file_path = RecordFilePathBuilder.build(
            test_module_path=test_module_path,
            test_function=test_function,
        )

        if (RecordType.all in record_type or RecordType.http in record_type) and not (
            record_file_path.exists() and record_no_overwrite
        ):
            record_file_path.unlink(missing_ok=True)

            vcr_object = vcr.VCR(
                cassette_library_dir=str(record_file_path.parent),
                record_mode="once",
                **vcr_config,
            )
            vcr_object.register_persister(VCRFilesystemPersister)

            with vcr_object.use_cassette(record_file_path.name) as cassette:
                yield cassette
        elif record_file_path.exists():
            vcr_object = vcr.VCR(
                cassette_library_dir=str(record_file_path.parent),
                record_mode="none",
                **vcr_config,
            )
            vcr_object.register_persister(VCRFilesystemPersister)

            with vcr_object.use_cassette(record_file_path.name) as cassette:
                yield cassette
        else:
            raise AttributeError(
                f"No comparison possible since there is no vcr cassette : {record_file_path}",
            )
    else:
        yield None


record_fixture = pytest.fixture(name="record_http", autouse=True)(
    record_http_context_manager
)



# IMPORT STANDARD
import json
import logging
from copy import deepcopy
from hashlib import sha256
from pathlib import Path
from typing import Any, List

# IMPORT THIRD-PARTY
import pytest
from _pytest.fixtures import SubRequest

# IMPORT INTERNAL
from pytest_recorder.record_type import RecordType

logger = logging.getLogger(__name__)


class ObjectCollector:
    @property
    def hash_only(self) -> bool:
        return self._hash_only

    @hash_only.setter
    def hash_only(self, value: bool):
        self._hash_only = value

    @property
    def object_list(self) -> List[Any]:
        return deepcopy(self._object_list)

    def add_verify(self, obj: Any, *args, **kwargs):
        if hasattr(obj, "to_dict"):
            obj = obj.to_dict(*args, **kwargs)
        self._object_list.append(obj)

    def __init__(
        self,
        object_list: List = None,
        hash_only: bool = True,
    ) -> None:
        self._hash_only = hash_only
        self._object_list = object_list or []


class ObjectCollectorHandler:
    @staticmethod
    def jsonify_record_data(record_model: ObjectCollector) -> ObjectCollector:
        object_list = record_model.object_list
        data_str_list = [json.dumps(data, sort_keys=True) for data in object_list]

        record_model = ObjectCollector(
            object_list=data_str_list,
            hash_only=True,
        )
        return record_model

    @staticmethod
    def hash_record_data(record_model: ObjectCollector) -> ObjectCollector:
        object_list = record_model.object_list
        data_str_list = [json.dumps(data, sort_keys=True) for data in object_list]
        data_hash_list = [sha256(data.encode()).hexdigest() for data in data_str_list]

        record_model = ObjectCollector(
            object_list=data_hash_list,
            hash_only=True,
        )
        return record_model

    @classmethod
    def jsonify_and_hash_record_data(
        cls, record_model: ObjectCollector
    ) -> ObjectCollector:
        object_list = record_model.object_list
        object_json_list = [json.dumps(data, sort_keys=True) for data in object_list]

        if record_model.hash_only:
            object_hash_list = [
                sha256(obj.encode()).hexdigest() for obj in object_json_list
            ]
            record_model = ObjectCollector(
                object_list=object_hash_list,
                hash_only=True,
            )
        else:
            record_model = ObjectCollector(
                object_list=object_json_list,
                hash_only=False,
            )

        return record_model

    @classmethod
    def persist(cls, record_model: ObjectCollector, record_file_path: Path):
        logger.debug("Making record folder : %s", record_file_path.parent)
        record_file_path.parent.mkdir(parents=True, exist_ok=True)

        with record_file_path.open(mode="w", encoding="utf-8", newline="\n") as file:
            logger.debug("Writing record file : %s", record_file_path)
            json.dump(record_model.object_list, file)

    @staticmethod
    def load_record_model(
        hash_only: bool,
        record_file_path: Path,
    ) -> ObjectCollector:
        if record_file_path.exists():
            logger.debug("Loading record file : %s", record_file_path)
            with record_file_path.open(
                mode="r", encoding="utf-8", newline="\n"
            ) as file:
                object_list = json.load(file)
        else:
            raise AttributeError(
                f"Cannot load record file : {record_file_path}",
            )

        record_model_loaded = ObjectCollector(
            object_list=object_list,
            hash_only=hash_only,
        )

        return record_model_loaded

    @staticmethod
    def verify(
        record_current: ObjectCollector,
        record_loaded: ObjectCollector,
        record_file_path: Path,
    ):
        current_data_list = record_current.object_list
        loaded_data_list = record_loaded.object_list
        if len(current_data_list) != len(loaded_data_list):
            raise AttributeError(
                "Record have different number of objects:\n"
                f"\nRECORD_FILE_PATH =\n{record_file_path}\n"
                f"\nCURRENT_OBJECT_LIST = {len(current_data_list)}\n"
                f"\nLOADED_OBJECT_LIST  = {len(loaded_data_list)}\n"
            )

        for index, (current, loaded) in enumerate(
            zip(current_data_list, loaded_data_list)
        ):
            if current != loaded:
                raise AttributeError(
                    "Record's data have changed.\n"
                    f"\nRECORD_FILE_PATH =\n{record_file_path}\n"
                    f"\nINDEX = {index}\n"
                    f"\nCURRENT =\n{current}\n"
                    f"\nPREVIOUS =\n{loaded}\n"
                )


class RecordFilePathBuilder:
    @staticmethod
    def build(test_module_path: Path, test_function: str, hash_only: bool) -> Path:
        test_module = test_module_path.stem
        data_folder_name = "object_hash" if hash_only else RecordType.object.name
        data_file_folder_path = test_module_path.parent / "record" / data_folder_name
        data_file_name = f"{test_function}.json"
        data_file_path = data_file_folder_path / test_module / data_file_name

        return data_file_path


def record_context_manager(
    request: SubRequest,
):
    """
    Provides an ObjectCollector.

    ObjectCollector allows collecting object.

    Once the object are collected they are:
    - Either saved in a file.
    - Either compared to the last saved objects.
    """

    record_no_overwrite = request.config.getoption("--record-no-overwrite")
    record_no_hash = request.config.getoption("--record-no-hash")
    record_type = request.config.getoption("--record")
    test_function = request.node.name
    test_module_path = Path(request.node.fspath)  # PYTEST 6.2.2 COMPATIBILITY

    record = ObjectCollector()

    yield record

    if record_no_hash:
        record.hash_only = False

    record_file_path = RecordFilePathBuilder.build(
        test_module_path=test_module_path,
        test_function=test_function,
        hash_only=record.hash_only,
    )

    record_formatted = ObjectCollectorHandler.jsonify_and_hash_record_data(
        record_model=record,
    )

    if (RecordType.all in record_type or RecordType.object in record_type) and not (
        record_file_path.exists() and record_no_overwrite
    ):
        ObjectCollectorHandler.persist(
            record_file_path=record_file_path,
            record_model=record_formatted,
        )
    elif record_file_path.exists():
        record_loaded = ObjectCollectorHandler.load_record_model(
            hash_only=record.hash_only,
            record_file_path=record_file_path,
        )

        ObjectCollectorHandler.verify(
            record_current=record_formatted,
            record_loaded=record_loaded,
            record_file_path=record_file_path,
        )
    else:
        raise AttributeError(
            f"No record to compare with the collected objects : {record_file_path}",
        )


record_fixture = pytest.fixture(name="record")(record_context_manager)
